package net.sf.nfp.mini.analize;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import javax.microedition.rms.InvalidRecordIDException;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreNotOpenException;

public class Statistics {
	private int earliestHighTemp = Integer.MAX_VALUE;
	private int shortestPeriod = Integer.MAX_VALUE;
	private int periodCount = 0;
	private RecordStore statistics;
	private static final String RMS_STAT = "stat";

	public Statistics() throws RecordStoreException, IOException {
		statistics = RecordStore.openRecordStore(RMS_STAT, true);
		if (statistics.getNumRecords() != 0) 
			read();
	}

	private void read() throws RecordStoreNotOpenException,
			InvalidRecordIDException, RecordStoreException, IOException {
		byte[] record = statistics.getRecord(getRecordId());
		DataInputStream stream = new DataInputStream(new ByteArrayInputStream(
				record));
		earliestHighTemp = stream.readInt();
		shortestPeriod = stream.readInt();
		periodCount = stream.readInt();
	}

	public int getEarliestHighTemp() {
		return earliestHighTemp;
	}

	public int getPeriodCount() {
		return periodCount;
	}

	public int getShortestPeriod() {
		return shortestPeriod;
	}

	public void write() throws IOException, RecordStoreException,
			RecordStoreException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream stream = new DataOutputStream(baos);
		stream.writeInt(earliestHighTemp);
		stream.writeInt(shortestPeriod);
		stream.writeInt(periodCount);
		byte[] bytes = baos.toByteArray();
		if(statistics.getNumRecords() == 0)
			statistics.addRecord(bytes, 0, bytes.length);
		else
			statistics.setRecord(getRecordId(), bytes, 0, bytes.length);
	}

	private int getRecordId() throws RecordStoreNotOpenException, RecordStoreException {
		return statistics.enumerateRecords(null,null,false).nextRecordId();
	}

	private static Statistics instance = null;

	public static Statistics instance() throws RecordStoreException,
			IOException {
		if (instance == null)
			instance = new Statistics();
		return instance;
	}

//	public void update(Period period) throws IOException, RecordStoreException {
//		if (period.getDays() < shortestPeriod) {
//			shortestPeriod = period.getDays();
//			write();
//		}
//	}
	
}
