package net.sf.nfp.mini.dao;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;

import net.sf.nfp.mini.data.Persistent;

public abstract class AbstractDAO {
	protected abstract RecordStore getBaseRMS() throws RecordStoreException;

	protected RecordStore lazyOpenRMS(RecordStore rms, String name) throws RecordStoreException {
		if (rms == null)
			rms = RecordStore.openRecordStore(name, true);
		return rms;
	}

	protected Persistent find(int id, Persistent persistent) throws IOException, RecordStoreException {
		byte[] bytes = getBaseRMS().getRecord(id);
		persistent = AbstractDAO.convert(persistent, bytes);
		persistent.setId(id);
		return persistent;
	}

	protected Persistent persist(Persistent persistent) throws RecordStoreException, IOException {
		byte[] bytes = AbstractDAO.convert(persistent);
		if(persistent.getId() == 0) {
			int id = getBaseRMS().addRecord(bytes, 0, bytes.length);
			persistent.setId(id);
		}else{
			getBaseRMS().setRecord(persistent.getId(), bytes, 0, bytes.length);
		}
		return persistent;
	}

	public abstract void close() throws RecordStoreException;

	public void delete(int id) throws RecordStoreException {
		getBaseRMS().deleteRecord(id);
	}

	public static Persistent convert(Persistent persistent, byte[] bytes) throws IOException {
		DataInputStream in = new DataInputStream(new ByteArrayInputStream(bytes));
		try {
			persistent.readFrom(in);
		}finally{
			in.close();
		}
		return persistent;
	}

	public static byte[] convert(Persistent persistent) throws IOException {
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		DataOutputStream out = new DataOutputStream(bout);
		try {
			persistent.writeTo(out);
		}finally {
			out.close();
		}
		return bout.toByteArray();		
	}
	
	public static void deleteAll(RecordStore store) throws RecordStoreException {
		String name = store.getName();
		store.closeRecordStore();
		RecordStore.deleteRecordStore(name);
	}
}
