package net.sf.nfp.mini.data;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.Calendar;
import java.util.Date;
import java.util.Vector;

import javax.microedition.rms.RecordStoreException;

import net.sf.log.mobile.Log;
import net.sf.nfp.mini.dao.LogicDAO;
import net.sf.nfp.mini.dao.ObservationDAO;
import net.sf.nfp.mini.dao.PeriodDAO;
import net.sf.nfp.mini.main.NFPControler;
import net.sf.nfp.mini.misc.Utils;

public class Exporter {
	/*
	 * 0 - period number 1 - period day 2 - date & time 3 - temperature 4 -
	 * mucus category 5 - mucus description 6 - is bleeding 7 - is disturbed
	 */
	public void exportToCSV(OutputStream stream, NFPControler controler) throws IOException, RecordStoreException {
		PrintStream writer = new PrintStream(stream);
		PeriodDAO periodDAO = controler.getPeriodDAO();
		ObservationDAO observationDAO = controler.getObservationDAO();
		try {
			String[] header = {
				"${csv.period}",
				"${csv.observation}",
				"${csv.date}",
				"${csv.temperature}",
				"${csv.mucus.id}",
				"${csv.mucus.category}",
				"${csv.mucus.description}",
				"${csv.bleeding}",
				"${csv.disturbed}"
			};
			for(int i=0;i<header.length;++i) {
				writer.print('"');
				writer.print(header[i]);
				writer.print('"');
				if(i < header.length - 1)
					writer.print(',');
			}
			writer.println();
			Vector periods = periodDAO.findAll();
			controler.progressListner.setMaxValue(periods.size() * 100);
			for (int i = 0; i < periods.size(); ++i) {
				Period period = (Period) periods.elementAt(i);
				Vector ids = periodDAO.getObservationIds(period);
				for (int j = 0; j < ids.size(); ++j) {
					controler.progressListner.setValue(i * 100 + (j * 100 / ids.size()));
					Integer id = (Integer) ids.elementAt(j);
					Observation ob = observationDAO.find(id.intValue());
					if (ob == null)
						continue;
					writer.print(i + 1);
					writer.print(',');
					writer.print(j + 1);
					writer.print(",\"");
					writer.print(toCSVString(ob.getDate()));
					writer.print("\",");
					writer.print(Utils.formatTemperature(ob.getTemperature()));
					writer.print(',');
					writer.print(ob.getMucus().getId());
					writer.print(',');
					writer.print(ob.getMucus().getCategory());
					writer.print(',');
					writer.print(ob.getMucus().toString());
					writer.print(',');
					writer.print(ob.isBleeding() ? '1' : '0');
					writer.print(',');
					writer.print(ob.isDisturbed() ? '1' : '0');
					writer.print(',');
					writer.print(ob.getNotes());
					writer.println();
					if((i+j) % 10 == 0)
						writer.flush();
				}
			}
		} finally {
			writer.close();
			stream.close();
		}
	}

	private static String toCSVString(Date date) {
		if (date == null)
			return "";
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		return Utils.zeroPadded(calendar.get(Calendar.YEAR), 4) + "."
				+ Utils.zeroPadded(calendar.get(Calendar.MONTH) + 1, 2) + "."
				+ Utils.zeroPadded(calendar.get(Calendar.DAY_OF_MONTH), 2) + " "
				+ Utils.zeroPadded(calendar.get(Calendar.HOUR_OF_DAY), 2) + ":"
				+ Utils.zeroPadded(calendar.get(Calendar.MINUTE), 2);
	}

	/*
	 * 0 - period number 1 - period day 2 - date & time 3 - temperature 4 -
	 * mucus category 5 - mucus description 6 - is bleeding 7 - is disturbed
	 */	
	public void importFromCSV(InputStream stream, long size, NFPControler controler) throws Exception {		
		Log.log("Exporter.importFromCSV() "+stream.available());
		InputStreamReader reader = new InputStreamReader(stream);
		Log.log("Exporter.importFromCSV() "+stream.available());
		PeriodDAO periodDAO = controler.getPeriodDAO();
		ObservationDAO observationDAO = controler.getObservationDAO();
		Period period = null;
		controler.progressListner.setMaxValue((int) size);
		long left = size;
		try {
			left -= readLine(reader).length(); // read CSV header
			periodDAO.deleteAll();
			observationDAO.deleteAll();
			String currentPeriod = null;
			String[] fields = new String[10];
			for (int lineNo = 1;; ++lineNo) {
				StringBuffer line = readLine(reader);
				if (line == null)
					break;
				left -= line.length();
				for (int i = 0; i < fields.length; ++i)
					fields[i] = grabField(line);
				if (!fields[0].equals(currentPeriod)) {
					if (period != null)
						periodDAO.persist(period);
					period = new Period();
					currentPeriod = fields[0];
				}
				Observation observation = new Observation(
						parseCSVDate(fields[2]), 
						parseCSVTemperature(fields[3]),
						MucusRegistry.instance().find(Integer.parseInt(fields[4])),
						parseCSVBoolean(fields[7]),
						parseCSVBoolean(fields[8]),
						fields[9]);
				
				LogicDAO.addObservationToPeriod(observation, new Period[]{period}, controler);
				controler.progressListner.setValue((int) (size - left));
			}
		} finally {
			reader.close();
			stream.close();
		}
		if(period != null)
			periodDAO.setCurrent(period);
	}

	private static int parseCSVTemperature(String string) {
		if ("".equals(string))
			return Observation.NO_TEMPERATURE;
		return Utils.parseTemperature(string);
	}

	private static boolean parseCSVBoolean(String string) {
		return "1".equals(string.toLowerCase());
	}

	private static Date parseCSVDate(String string) {
		// dd.mm.yyyy hh:mm
		// yyyy.mm.dd hh:mm
		// 01234567890123457
		final int year = Integer.parseInt(string.substring(0, 4));
		final int month = Integer.parseInt(string.substring(5, 7));
		final int day = Integer.parseInt(string.substring(8, 10));
		final int hour = Integer.parseInt(string.substring(11, 13));
		final int minute = Integer.parseInt(string.substring(14, 16));
		Calendar calendar = Calendar.getInstance();
		calendar.set(Calendar.DAY_OF_MONTH, day);
		calendar.set(Calendar.MONTH, month - 1);
		calendar.set(Calendar.YEAR, year);
		calendar.set(Calendar.HOUR, hour);
		calendar.set(Calendar.MINUTE, minute);
		return calendar.getTime();
	}

	private static String grabField(StringBuffer line) {
		String field;
		if (line.length() == 0)
			return "";
		if (line.charAt(0) == '"') {
			line.delete(0, 1);// delete '"'
			field = readUntil(line, '"');
			line.delete(0, 1);// delete '"'
		} else
			field = readUntil(line, ',');
		Log.log("grabField=" + field);
		return field;
	}

	private static String readUntil(StringBuffer line, char delim) {
		StringBuffer field = new StringBuffer();
		for (int i = 0; i < line.length(); ++i) {
			char c = line.charAt(i);
			if (c == delim) {
				line.delete(0, i + 1);
				break;
			}
			field.append(c);
		}
		return field.toString().trim();
	}

	private static StringBuffer readLine(InputStreamReader reader) throws IOException {
		StringBuffer line = new StringBuffer();
		for (;;) {
			int c = reader.read();
			if (c == -1)
				return line.length() == 0 ? null : line;
			if (c == '\n')
				return line;
			line.append((char) c);
		}
	}

}
