/* 
 * Natural Family Planning for J2ME
 * Copyright (C) 2007 Krzysztof Rzymkowski <rzymek@users.sourceforge.net>
 * 	
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */ 
 
package net.sf.nfp.mini.data;


import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Date;

import net.sf.nfp.mini.misc.Utils;

public class Observation implements Persistent {
	public static final byte MUCUS_NONE = 0;
	public static final byte MUCUS_POOR = 1;
	public static final byte MUCUS_FINE = 2;
	
	private int id;
	private Date date;
	/** Temperature in 0,01 degrees (eg. 36,6 = 3660) */
	private short temperature;
	private boolean disturbed;
	private boolean bleeding;
	private String notes;
	private Mucus mucus; 
	
	public static final int MIN_TEMP = 3600;
	public static final int MAX_TEMP = 3760;

	public static final byte NO_TEMPERATURE = 0;
	
	public Observation() {
		mucus = MucusRegistry.instance().find(1);
		date = new Date();
		notes = "";
	}

	public Observation(Date date, int temperature, Mucus mucus, 
			boolean period, boolean disturbed, String notes) {		
		this.date = Utils.assignDate(date);
		this.temperature = (short) temperature;
		this.mucus = mucus;
		this.disturbed = disturbed;
		this.bleeding = period;
		this.notes = notes;
	}

	public boolean isBleeding() {
		return bleeding;
	}
	public int getTemperature() {
		return temperature;
	}
	public boolean hasTemperature() {
		return temperature != NO_TEMPERATURE;
	}

	public Mucus getMucus() {
		return mucus;
	}
	public boolean isDisturbed() {
		return disturbed;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public Date getDate() {
		return date;
	}

	public String toString() {
		StringBuffer s = new StringBuffer("[");
		s.append(Utils.formatDate(date));
		s.append(' ');
		if(isBleeding())
			s.append("${bleeding}, ");
		else
			s.append(Utils.formatTemperature(getTemperature())).append(", ");
		s.append(mucus);
		s.append(']');
		return s.toString();
	}

	public void writeTo(DataOutputStream out) throws IOException {
		out.writeLong(date.getTime());
		out.writeShort(temperature);
		out.writeInt(mucus.getId());
		out.writeBoolean(disturbed);
		out.writeBoolean(bleeding);
		out.writeUTF(notes);
	}

	public void readFrom(DataInputStream in) throws IOException {
		date = new Date(in.readLong());
		temperature = in.readShort();
		mucus = MucusRegistry.instance().find(in.readInt());
		disturbed = in.readBoolean();
		bleeding = in.readBoolean();
		notes = in.readUTF();
	}

	public String getNotes() {
		return notes;
	}
}
