/* 
 * Natural Family Planning for J2ME
 * Copyright (C) 2007 Krzysztof Rzymkowski <rzymek@users.sourceforge.net>
 * 	
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package net.sf.nfp.mini.data;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Date;

import net.sf.nfp.mini.misc.Utils;

public class Period implements Persistent {
	private int id = 0;
	private Date start = null;
	private Date end = null;
	private int dataId = 0;

	public Period() {
	}

	public Period(Date date) {
		start = Utils.assignDate(date);
		end = Utils.assignDate(date);
	}
	
	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}
	
	public int getDataId() {
		return dataId;
	}

	public void setDataId(int dataId) {
		this.dataId = dataId;
	}

	public Date getStart() {
		return start;
	}
	public Date getEnd() {
		return end;
	}
	
	public void readFrom(DataInputStream in) throws IOException {
		start = new Date(in.readLong());
		end = new Date(in.readLong());
		dataId = in.readInt();
	}

	public void writeTo(DataOutputStream out) throws IOException {
		out.writeLong(start.getTime());
		out.writeLong(end.getTime());
		out.writeInt(dataId);
	}

	public void setEnd(Date end) {
		this.end = Utils.assignDate(end);
	}

	public void setStart(Date start) {
		this.start = Utils.assignDate(start);
	}

	public String toString() {
		return Utils.formatDate(start) + "-" + Utils.formatDate(end);
	}
	
	public boolean equals(Object arg) {
		if (arg instanceof Period) {
			Period p = (Period) arg;
			if(p == null)
				return false;
			return p.getId() == id;			
		}
		return false;
	}
}
