/* 
 * Natural Family Planning for J2ME
 * Copyright (C) 2007 Krzysztof Rzymkowski <rzymek@users.sourceforge.net>
 * 	
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package net.sf.nfp.mini.misc;

import java.util.Calendar;
import java.util.Date;

import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.List;

public class Utils {

	public static String zeroPadded(int number, int count) {
		StringBuffer buf = new StringBuffer(Integer.toString(number));
		while (buf.length() < count)
			buf.insert(0, '0');
		return buf.toString();
	}

	public static int toMinutes(long time) {
		return (int) (time / 60 /* sec *// 1000 /* ms */);
	}

	public static int toDays(long time) {
		return toMinutes(time) / 60/* min *// 24/* h */;
	}

	public static int getDaysBetween(Date begin, Date end) {
		return toDays(end.getTime() - begin.getTime());
	}

	public static String formatDate(Date date) {
		if (date == null)
			return "";
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		return Utils.zeroPadded(calendar.get(Calendar.DAY_OF_MONTH), 2) + "."
				+ Utils.zeroPadded(calendar.get(Calendar.MONTH) + 1, 2);
	}

	public static String formatTemperature(int i) {
		int integral = i / 100;
		int fractional = i - integral * 100;
		return integral + "." + zeroPadded(fractional, 2);
	}

	public static boolean equals(Date date1, Date date2) {
		return toDays(date1.getTime()) == toDays(date2.getTime());
	}

	public static final int DIRECTION_LEFT = -1;
	public static final int DIRECTION_RIGHT = 1;
	
	public static Date assignDate(Date date) {
		//workaroung Nokia JVM bug
		return new Date(date.getTime());		
	}

	public static int parseTemperature(String tempString) {
		int dotIndex = tempString.indexOf('.');
		if (dotIndex == -1) {
			return Integer.parseInt(tempString) * 100;
		} else {
			int integral = Integer.parseInt(tempString.substring(0, dotIndex));
			String factoralString = tempString.substring(dotIndex + 1);
			while (factoralString.length() < 2)
				factoralString += "0";
			if (factoralString.length() > 2)
				factoralString.substring(0, 2);
			int factoral = Integer.parseInt(factoralString);
			return integral * 100 + factoral;
		}
	}

	public static void deleteAll(List choise) {
		//#ifndef midp1.0
		choise.deleteAll();
		//#else		
		while(choise.size()>0)
			choise.delete(0);
		//#endif				
	}
	public static void deleteAll(ChoiceGroup choise) {
		//#ifndef midp1.0
		choise.deleteAll();
		//#else		
		while(choise.size()>0)
			choise.delete(0);
		//#endif				
	}
}
