package net.sf.nfp.mini.model;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Vector;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.List;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;

import net.sf.mvc.mobile.Navigation;
import net.sf.mvc.mobile.command.ActionCommand;
import net.sf.nfp.mini.data.Mucus;
import net.sf.nfp.mini.data.MucusRegistry;
import net.sf.nfp.mini.main.NFPControler;
import net.sf.nfp.mini.misc.MucusUtils;
import net.sf.nfp.mini.misc.Utils;
import net.sf.nfp.mini.view.MucusEditorView;

public class MucusListModel extends CommonModel {
	private List list;
	private RecordStore rms;
	private Vector registry;

	public MucusListModel(NFPControler controler) {
		super(controler);		
	}

	public void load(Object parameter) throws Exception {
		if(parameter == null) {
			try {
				rms = RecordStore.openRecordStore(MucusRegistry.MUCUS_REGISTRY,	true);
				if (rms.getNumRecords() == 0)
					saveDefaults();
				refresh();
			} catch (Exception e) {
				e.printStackTrace();
				throw new RuntimeException(e.toString());
			}
		}else{
			Mucus mucus = (Mucus) parameter;
			save(mucus);
			refresh();
		}
	}

	public void setView(Displayable v) {
		this.list = (List) v;
		list.addCommand(new ActionCommand("${edit}", Command.ITEM, 1) {
			public Navigation execute(Displayable d) throws Exception {
				Mucus mucus = (Mucus) registry.elementAt(list
						.getSelectedIndex());
				return new Navigation("edit-mucus", mucus);
			}
		});
		list.addCommand(new ActionCommand("${add}", Command.SCREEN, 2) {
			public Navigation execute(Displayable d) throws Exception {
				return new Navigation("edit-mucus", new Mucus());
			}
		});
		list.addCommand(new ActionCommand("${remove}", Command.ITEM, 3) {
			public Navigation execute(Displayable d) throws Exception {
				Mucus mucus = (Mucus) registry.elementAt(list
						.getSelectedIndex());
				rms.deleteRecord(mucus.getId());
				refresh();
				return null;
			}
		});
		list.addCommand(new ActionCommand("${default}", Command.ITEM, 3) {
			public Navigation execute(Displayable d) throws Exception {
				rms.closeRecordStore();
				MucusRegistry.close();
				RecordStore.deleteRecordStore(MucusRegistry.MUCUS_REGISTRY);
				rms = RecordStore.openRecordStore(MucusRegistry.MUCUS_REGISTRY,
						true);
				saveDefaults();
				refresh();
				return null;
			}
		});
		list.addCommand(new ActionCommand("${back}", Command.BACK, 10) {
			public Navigation execute(Displayable d) throws Exception {
				MucusRegistry.close();
				return new Navigation("input", null);
			}
		});
	}

	private void refresh() throws RecordStoreException, IOException {
		Utils.deleteAll(list);
		registry = MucusUtils.loadFromRMS(rms);
		for (int i = 0; i < registry.size(); ++i) {
			Mucus m = (Mucus) registry.elementAt(i);
			System.out.println("mucus=" + i + ": " + m.toString());
			list.append(m.getId() + ": ["
					+ MucusEditorView.CATEGORY_NAMES[m.getCategory()] + "] "
					+ m.toString(), 
					MucusRegistry.instance().getImage(m));
		}
	}

	private void saveDefaults() throws IOException, RecordStoreException {
		registry = MucusUtils.createDefaultsVector();
		for (int i = 0; i < registry.size(); ++i) {
			Mucus mucus = (Mucus) registry.elementAt(i);
			mucus.setId(0);
			save(mucus);
		}
	}

	private void save(Mucus mucus) throws IOException, RecordStoreException {
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		DataOutputStream out = new DataOutputStream(bout);
		mucus.writeTo(out);
		out.close();
		byte[] data = bout.toByteArray();
		if (mucus.getId() == 0) {
			int id = rms.addRecord(data, 0, data.length);
			mucus.setId(id);
		} else {
			rms.setRecord(mucus.getId(), data, 0, data.length);
		}
	}
}
