package net.sf.nfp.mini.view;

import java.util.Calendar;
import java.util.Date;
import java.util.Vector;

import javax.microedition.lcdui.Graphics;

import net.sf.log.mobile.Log;
import net.sf.nfp.mini.data.Period;
import net.sf.nfp.mini.misc.Color;
import net.sf.nfp.mini.misc.Utils;

public class HistoryView extends PropagatingCanvas {
	private static final int SELECTED_COLOR = 0xff0000;
	private int boxWidth;
	private int boxHeight;
	private Vector periods;
	private int selected = 0;
	private static int[] COLORS = { 0xffff80, 0x80ff80, 0x8080ff };

	public HistoryView() {
		boxWidth = getWidth() / 3;
		boxHeight = getHeight() / 4;
	}

	protected void safePaint(Graphics g) {
		Log.log("HistoryView.safePaint()");
		clear(g);
		for (int i = 0; i < periods.size(); ++i) {
			if (i == selected)
				continue;
			g.setColor(COLORS[i % COLORS.length]);
			drawPeriod(g, (Period) periods.elementAt(i), i % 2 == 0);
		}
		g.setColor(SELECTED_COLOR);
		drawPeriod(g, (Period) periods.elementAt(selected), selected % 2 == 0);
		drawGrid(g);
	}

	private void drawGrid(Graphics g) {
		Log.log("HistoryView.drawGrid()");
		g.setColor(0x000000);
		String[][] labels = { 
			{ "I", "II", "III" }, 
			{ "IV", "V", "VI" }, 
			{ "VII", "VIII", "IX" },
			{ "X", "XI", "XII" },
		};
		for (int y = 0; y < 4; ++y)
			for (int x = 0; x < 3; ++x) {
				int gx = x * boxWidth;
				int gy = y * boxHeight;
				g.drawRect(gx, gy, boxWidth, boxHeight);
				drawString(g, labels[y][x], gx + boxWidth / 2, gy + boxHeight / 2);
			}
	}

	public void drawPeriod(Graphics g, Period period, boolean type) {
		Log.log("HistoryView.drawPeriod()");
		int[] start = split(period.getStart());
		int days = Utils.getDaysBetween(period.getStart(), period.getEnd()) + 1;
		int row = start[1] / 3;
		int col = start[1] % 3;
		int day = start[2] * boxWidth / 30;
		if (day > boxWidth)
			day = boxWidth;
		int width = days * boxWidth / 30;
		int offset = boxHeight / 6;
		int height = boxHeight - offset;
		if (type)
			offset = 0;
		g.fillRect(col * boxWidth + day, row * boxHeight + offset, width, height);
	}

	public int[] split(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		return new int[] { cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), cal.get(Calendar.DAY_OF_MONTH), };
	}

	private void clear(Graphics g) {
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, getWidth(), getHeight());
		g.setColor(Color.BLACK);
	}

	private void drawString(Graphics g, String str, int x, int y) {
		int strWidth = g.getFont().stringWidth(str);
		int strHeight = g.getFont().getHeight();
		g.drawString(str, x - strWidth / 2, y + strHeight / 2, Graphics.LEFT | Graphics.BASELINE);
	}

	public Period getSelectedPeriod() {
		return (Period) periods.elementAt(selected);
	}

	public void setPeriods(Vector periods) {
		this.periods = periods;
	}

	public void setSelectedPeriod(Period selected) {
		for(int i=0;i<periods.size();++i) {
			Period period = (Period) periods.elementAt(i);
			if(period.equals(selected))
				this.selected = i;
		}			
	}
	public void selectNext(int direction) {
		int next = selected - direction;
		if (0 <= next && next < periods.size())
			selected = next;
		repaint();
	}
}
